@@ -75,6 +75,80 @@ class Expression(_Ast, ast_utils.AsList):
         return msg
 
 
+@dataclass
+class BinaryExpression(_Ast):
+    left: _Ast
+    right: Optional[_Ast] = None
+    operator: Optional[str] = None  # This will be overridden
+
+    def __str__(self) -> str:
+        if self.right is not None:
+            return f"{self.left} {self.operator} {self.right}"
+        else:
+            return f"{self.left}"
+
+
+@dataclass
+class ExpressionLt(BinaryExpression):
+    operator: str = "<"
+
+
+@dataclass
+class ExpressionAdd(BinaryExpression):
+    operator: str = "+"
+
+
+@dataclass
+class ExpressionSub(BinaryExpression):
+    operator: str = "-"
+
+
+@dataclass
+class ExpressionMul(BinaryExpression):
+    operator: str = "*"
+
+
+# Unfortunately we have to recreate the expression based on the parsing when creating the sql.
+# So we need to know if the original had parentheses or not, they will all inherit from this class.
+class ParensExpression(BinaryExpression):
+    pass
+
+
+@dataclass
+class ExpressionAddParens(ParensExpression):
+    operator: str = "+"
+
+
+@dataclass
+class ExpressionSubParens(ParensExpression):
+    operator: str = "-"
+
+
+@dataclass
+class ExpressionMulParens(ParensExpression):
+    operator: str = "*"
+
+
+@dataclass
+class ExpressionDivParens(ParensExpression):
+    operator: str = "/"
+
+
+@dataclass
+class ExpressionDiv(BinaryExpression):
+    operator: str = "/"
+
+
+@dataclass
+class ExpressionGt(BinaryExpression):
+    operator: str = ">"
+
+
+@dataclass
+class ExpressionEq(BinaryExpression):
+    operator: str = "="
+
+
 @dataclass
 class _JoinType(_Ast):
     def __str__(self) -> str:
@@ -145,18 +219,42 @@ class Join(_Ast):
         else:
             return None
 
-    #
-    # def __post_init__(self) -> None:
-    #     if isinstance(self.name, JoinType):
-    #         temp = self.join_type
-    #         self.join_type = self.name
-    #         self.name = temp  # type: ignore[assignment]
-    #     if isinstance(self.join_type, Name):
-    #         # Now we need to shift everything , since join_type is now our left_id
-    #         temp = self.left_id
-    #         self.left_id = self.join_type
-    #         self.right_id = temp
-    #         self.join_type = None
+
+@dataclass
+class _FileType(_Ast):
+    pass
+
+
+@dataclass
+class Csv(_FileType):
+    def __str__(self) -> Literal["csv"]:  # type: ignore[name-defined]
+        return "csv"
+
+
+@dataclass
+class Tsv(_FileType):
+    def __str__(self) -> Literal["tsv"]:  # type: ignore[name-defined]
+        return "tsv"
+
+
+@dataclass
+class FileType(_FileType):
+    file_type: _FileType
+
+    def __str__(self) -> str:
+        return str(self.file_type)
+
+
+@dataclass
+class To(_Ast):
+    file_type: _FileType
+    name: Name
+
+    def __str__(self) -> str:
+        return f"TO {self.file_type} {self.name}"
+
+    def get_file_type(self) -> _FileType:
+        return self.file_type
 
 
 @dataclass
@@ -212,7 +310,7 @@ class Select(_Ast):
 
 @dataclass
 class DeriveBody(_Ast):
-    val: Union[str, Expression]
+    val: Union[str, _Ast]
 
     def __str__(self) -> str:
         return str(self.val)
@@ -292,13 +390,24 @@ class Descending(_Direction):
 
 @dataclass
 class Sort(_Ast):
-    fields: SortFields
-    direction: Optional[Direction] = None
+    direction1: Optional[Direction] = None
+    fields: Optional[SortFields] = None
+    direction2: Optional[Direction] = None
+
+    def get_direction(self) -> Optional[Direction]:
+        ret = None
+        if self.direction1 is not None:
+            ret = self.direction1
+        elif self.direction2 is not None:
+            ret = self.direction2
+        return ret
 
     def __str__(self) -> str:
-        return (
-            f'{str(self.fields)} {self.direction if self.direction is not None else ""}'
-        )
+        direction = self.get_direction()
+        ret = f"{str(self.fields)}"
+        if direction is not None:
+            ret = f"{ret} {str(direction)}"
+        return ret
 
 
 @dataclass
@@ -433,10 +542,10 @@ class Root(_Ast):
 @dataclass
 class NameValuePair(_Ast):
     name: str
-    value: str
+    value: _Ast
 
     def __str__(self) -> str:
-        return f"({self.value})"
+        return f"{self.value}"
 
 
 class ToAst(Transformer):
@@ -489,7 +598,6 @@ def to_sql(prql: str, verbose: bool = False) -> str:
         rich.print(ast.get_from())
     if STDLIB_AST is None:
         STDLIB_AST = parse(read_file("stdlib.prql"))
-    # @formatter:off
     return (
         ast_to_sql(ast.get_from(), [ast, STDLIB_AST], verbose=verbose)
         .replace("   ", " ")
@@ -553,7 +661,7 @@ def _generate_alias(s: str, n: int = 1) -> str:
 
 
 @enforce_types
-def replace_all_tables_standalone(
+def replace_all_tables(
     from_long: str, from_short: str, join_long: List[str], join_short: List[str], s: str
 ) -> str:
     s = s.replace(f"{from_long}.", f"{from_short}.")
@@ -568,31 +676,7 @@ def wrap_replace_all_tables(
     from_long: str, from_short: str, join_long: List[str], join_short: List[str]
 ) -> Callable[[str], str]:
     def inner(x: str) -> str:
-        return replace_all_tables_standalone(
-            from_long, from_short, join_long, join_short, x
-        )
-
-    return inner
-
-
-@enforce_types
-def replace_tables_standalone(
-    from_long: str, from_short: str, join_long: str, join_short: str, s: str
-) -> str:
-    s = s.replace(f"{from_long}.", f"{from_short}.")
-    if join_long:
-        s = s.replace(f"{join_long}.", f"{join_short}.")
-    return s
-
-
-@enforce_types
-def wrap_replace_tables(
-    from_long: str, from_short: str, join_long: str, join_short: str
-) -> Callable[[str], str]:
-    def inner(x: str) -> str:
-        return replace_tables_standalone(
-            from_long, from_short, join_long, join_short, x
-        )
+        return replace_all_tables(from_long, from_short, join_long, join_short, x)
 
     return inner
 
@@ -622,32 +706,51 @@ class PRQLException(Exception):
     pass
 
 
+def _replace_variables(ast: _Ast, symbol_table: Dict[str, List[_Ast]]) -> str:
+    if isinstance(ast, Expression):
+        msg = ""
+        exp: Expression = ast
+        for s in exp.statements:
+            msg += str(replace_variables(str(s), symbol_table))
+        return msg
+    elif isinstance(ast, DeriveBody):
+        msg = _replace_variables(ast.val, symbol_table)  # type: ignore [arg-type]
+        return msg
+    elif isinstance(ast, BinaryExpression):
+        msg = ""
+        if ast.left is not None:
+            msg = _replace_variables(ast.left, symbol_table)
+        if ast.right is not None:
+            msg += ast.operator + _replace_variables(ast.right, symbol_table)
+
+        return msg
+    else:
+        key = str(ast)
+        if key in symbol_table:
+            return str(symbol_table[key][0])
+        else:
+            return str(ast)
+
+
 @enforce_types
 def replace_variables(_param: Any, symbol_table: Dict[str, List[_Ast]]) -> str:
-    param: str = str(_param)
-    if param in symbol_table:
-        if not symbol_table[param]:
+    param_str: str = str(_param)
+
+    if param_str in symbol_table:
+        if not symbol_table[param_str]:
             return _param
-        if isinstance(symbol_table[param][0], NameValuePair):
-            nvp: NameValuePair = symbol_table[param][0]
-            if isinstance(nvp.value, Expression):
-                msg = ""
-                exp: Expression = nvp.value
-                for s in exp.statements:
-                    msg += str(replace_variables(str(s), symbol_table))
-
-                return msg
-            if isinstance(nvp.value, DeriveBody):
-                msg = ""
-                body_exp: Expression = nvp.value.val  # type: ignore[assignment]
-                for s in body_exp.statements:
-                    msg += str(replace_variables(str(s), symbol_table))
-
-                return msg
+        if isinstance(symbol_table[param_str][0], NameValuePair):
+            nvp: NameValuePair = symbol_table[param_str][0]
+            msg = _replace_variables(nvp.value, symbol_table)
+            if msg is None:
+                return str(nvp.value)
             else:
-                return nvp.value
+                return msg
         else:
-            return str(symbol_table[param][0])
+            raise PRQLException(
+                f"Unkown type in symbol table , for {type(symbol_table[param_str][0])} "
+            )
+            # return str(symbol_table[param_str][0])
     else:
         return _param
 
@@ -676,58 +779,58 @@ def execute_function(
                 func_def = defintion
                 break
 
-    try:
-        if func_def:
-            for line in func_def.func_body.fields:
-                if isinstance(line, PipeBody):
-                    line = line.body
-                if isinstance(line, Expression):
-                    exp: Expression = line
-                    if len(exp.statements) != 1:
-                        raise PRQLException(
-                            "Currently only single statement functions that are text based are supported"
-                        )
-                    else:
-                        line = str(exp.statements[0])
-                if type(line) == str:
-                    args = {}
+    # try:
+    if func_def:
+        for line in func_def.func_body.fields:
+            if isinstance(line, PipeBody):
+                line = line.body
+            if isinstance(line, Expression):
+                exp: Expression = line
+                if len(exp.statements) != 1:
+                    raise PRQLException(
+                        "Currently only single statement functions that are text based are supported"
+                    )
+                else:
+                    line = str(exp.statements[0])
+            if type(line) == str:
+                args = {}
 
-                    vals = [
-                        safe_to_sql(
-                            replace_variables(f.parm1, symbol_table),
-                            roots,
-                            symbol_table,
-                        ),
-                        safe_to_sql(
-                            replace_variables(f.parm2, symbol_table),
-                            roots,
-                            symbol_table,
-                        ),
-                        safe_to_sql(
-                            replace_variables(f.parm3, symbol_table),
-                            roots,
-                            symbol_table,
-                        ),
-                    ]
+                vals = [
+                    safe_to_sql(
+                        replace_variables(f.parm1, symbol_table),
+                        roots,
+                        symbol_table,
+                    ),
+                    safe_to_sql(
+                        replace_variables(f.parm2, symbol_table),
+                        roots,
+                        symbol_table,
+                    ),
+                    safe_to_sql(
+                        replace_variables(f.parm3, symbol_table),
+                        roots,
+                        symbol_table,
+                    ),
+                ]
 
-                    if func_def.func_args is not None:
-                        for i in range(0, len(func_def.func_args.fields)):  # type: ignore[arg-type]
-                            n = str(func_def.func_args.fields[i])  # type: ignore[index]
-                            args[n] = vals[i]
-                        msg = line.format(**args)
-                    else:
-                        msg = line
-    except Exception as e:
-        msg = repr(e)
-        if "KeyError" in msg:
-            msg = f"Function {name} had an error executing , full error: {msg}"
-        raise PRQLException(msg)
+                if func_def.func_args is not None:
+                    for i in range(0, len(func_def.func_args.fields)):  # type: ignore[arg-type]
+                        n = str(func_def.func_args.fields[i])  # type: ignore[index]
+                        args[n] = vals[i]
+                    msg = line.format(**args)
+                else:
+                    msg = line
+    # except Exception as e:
+    #     msg = repr(e)
+    #     if "KeyError" in msg:
+    #         msg = f"Function {name} had an error executing , full error: {msg}"
+    #     raise PRQLException(msg)
     return msg
 
 
 @enforce_types
 def is_empty(a: Any) -> bool:
-    if a is not None and a != "None" and a != "none" and a != "":
+    if a is not None and a:
         return False
     return True
 
@@ -763,7 +866,7 @@ def safe_to_sql(
     elif isinstance(rule, Token) or isinstance(rule, Name):
         return str(rule)
     elif isinstance(rule, _Ast):
-        return ast_to_sql(rule, roots, symbol_table, verbose)
+        return ast_to_sql(rule, roots, symbol_table, None, verbose)
     # I've added the else clause explicitly to quiet mypy.
     # This is just an explicit statement of previous behaviour.
     else:
@@ -779,29 +882,56 @@ def safe_get_alias(join: Union[Join, From], join_long: str) -> str:
     return join_short
 
 
+def build_replace_tables(root: Root) -> Callable:
+    all_join_longs = []
+    all_join_shorts = []
+    _from = root.get_from()
+    from_long = str(_from.name)
+    from_short = safe_get_alias(_from, from_long)
+    joins: List[Join] = get_operation(_from.pipes.operations, Join, return_all=True)
+
+    for join in joins:
+        if join:
+            join_long = str(join.name)
+            join_short = safe_get_alias(join, join_long)
+
+            all_join_shorts.append(join_short)
+            all_join_longs.append(join_long)
+
+    replace_tables = wrap_replace_all_tables(
+        from_long, from_short, all_join_longs, all_join_shorts
+    )
+    return replace_tables
+
+
 @enforce_types
 def ast_to_sql(
     rule: Union[_Ast, Token],
     roots: Union[Root, List],
     symbol_table: Optional[Dict] = None,  # Optional[Dict[str, List[_Ast]]]
+    replace_tables: Optional[Callable] = None,
     verbose: bool = True,
 ) -> str:
+    # Renaming it here silences the static analysis warnings below for replace_tables being None
+    replace_tables_func: Callable = replace_tables  # type: ignore[assignment]
+
     if isinstance(roots, Root):
         root = roots
     else:
         root = roots[0]
 
-    if not symbol_table:
+    if symbol_table is None:
         symbol_table = build_symbol_table([roots] if isinstance(roots, Root) else roots)
-        # if verbose:
-        #     rich.print(symbol_table)
+
+    if replace_tables_func is None:
+        replace_tables_func: Callable = build_replace_tables(root)  # type: ignore[no-redef]
 
     if isinstance(rule, Token):
         return str(rule)
 
     if isinstance(rule, From):
         # The SQL template is in the form
-        # sql = f"SELECT {select_str} {agg_str} {derives_str} FROM {from_str} {join_str} WHERE {filter_str} {group_by_str} {havings_str} {order_by_str} {limit_str}"
+        # sql = f"SELECT {select_str} {agg_str} {derives_str} FROM {from_str} {join_str} WHERE {filter_str} {group_by_str} {havings_str} {order_by_str} {limit_str} {to_str}"
         # We will be creating these strings to form the final SQL,
         select_str = ""
         agg_str = ""
@@ -813,6 +943,7 @@ def ast_to_sql(
         havings_str = ""
         order_by_str = ""
         limit_str = ""
+        to_str = ""
         ###
 
         _from = rule
@@ -822,47 +953,30 @@ def ast_to_sql(
         joins: List[Join] = get_operation(ops.operations, Join, return_all=True)
         agg: Aggregate = get_operation(ops.operations, Aggregate)
         take: Take = get_operation(ops.operations, Take, last_match=True)
-        sort: Sort = get_operation(ops.operations, Sort, last_match=True)
+        sort: Sort = get_operation(ops.operations, Sort)
 
         filters = get_operation(
             ops.operations, Filter, return_all=True, before=Aggregate
         )
-        wheres_from_derives = get_operation(
-            ops.operations, Derive, return_all=True, before=Aggregate
-        )
+        wheres_from_derives = get_operation(ops.operations, Derive, return_all=True)
         havings = get_operation(
             ops.operations, Filter, return_all=True, after=Aggregate
         )
         selects = get_operation(ops.operations, Select, return_all=True)
+        tos = get_operation(ops.operations, To, last_match=True)
 
         from_long = str(_from.name)
         from_short = safe_get_alias(_from, from_long)
 
         from_str = f"`{from_long}`" + " " + from_short
-        all_join_longs = []
-        all_join_shorts = []
-        # We have to run these twice, once to create the replace_tables function then
-        # once to replace_tables within the definitions
 
         for join in joins:
             if join:
                 join_long = str(join.name)
                 join_short = safe_get_alias(join, join_long)
 
-                all_join_shorts.append(join_short)
-                all_join_longs.append(join_long)
-
-        replace_tables = wrap_replace_all_tables(
-            from_long, from_short, all_join_longs, all_join_shorts
-        )
-
-        for join in joins:
-            if join:
-                join_long = str(join.name)
-                join_short = safe_get_alias(join, join_long)
-
-                left_id = replace_tables(str(join.left_id))
-                right_id = replace_tables(str(join.right_id))
+                left_id = replace_tables_func(str(join.left_id))
+                right_id = replace_tables_func(str(join.right_id))
 
                 if right_id is None or right_id == "None":
                     right_id = left_id
@@ -877,13 +991,13 @@ def ast_to_sql(
                     )
 
                 if right_id.find(".") == -1:
-                    right_side = replace_tables(
+                    right_side = replace_tables_func(
                         str(join_long + "." + right_id).replace(
                             join_short + "." + join_short + ".", join_short + "."
                         )
                     )
                 else:
-                    right_side = replace_tables(
+                    right_side = replace_tables_func(
                         str(right_id).replace(
                             join_short + "." + join_short + ".", join_short + "."
                         )
@@ -895,30 +1009,41 @@ def ast_to_sql(
                 else:
                     join_type_str = str(join.get_join_type())
 
-                join_str += replace_tables(
+                join_str += replace_tables_func(
                     f"{join_type_str} {join.name} {join_short} ON {left_side} = {right_side} "
                 )
 
         if selects:
             for select in selects:
-                select_str += replace_tables(str(select))
+                select_str += replace_tables_func(str(select))
+
+        if tos:
+            # Tos is generated with `last_match=True`, so will always be single instance.
+            # Additionally, __str__method is defined for simplicity.
+            to_str += str(tos)
+
+        if havings:
+            havings_str = "HAVING "
+            for filter in havings:
+                if filter:
+                    for func in filter.fields:
+                        if func.val is not None:
+                            havings_str += (
+                                ast_to_sql(
+                                    rule=func.val,
+                                    roots=roots,
+                                    symbol_table=symbol_table,
+                                    replace_tables=replace_tables,
+                                    verbose=verbose,
+                                )
+                                + " AND "
+                            )
+            havings_str = havings_str.rstrip(" AND ")
 
         if agg:
 
             if agg.group_by is not None:
-                group_by_str = f"GROUP BY {replace_tables(str(agg.group_by))}"
-
-            if havings is not None and len(havings) > 0:
-                havings_str = "HAVING "
-                for filter in havings:
-                    if filter:
-                        for func in filter.fields:
-                            if func.val is not None:
-                                havings_str += (
-                                    ast_to_sql(func.val, roots, symbol_table, verbose)
-                                    + " AND "
-                                )
-                havings_str = havings_str.rstrip(" AND ")
+                group_by_str = f"GROUP BY {replace_tables_func(str(agg.group_by))}"
 
             upper = len(agg.aggregate_body.statements)
             i = 0
@@ -931,19 +1056,19 @@ def ast_to_sql(
                         function_call = agg.aggregate_body.statements[i]
 
                         if isinstance(function_call, FuncCall):
-                            agg_str += f"{ast_to_sql(function_call, roots, symbol_table, verbose)} as {name},"
+                            agg_str += f"{ast_to_sql(function_call, roots, symbol_table, replace_tables, verbose)} as {name},"
                         elif isinstance(function_call, str):
                             agg_str += f"{function_call} as {name},"
                         elif isinstance(function_call, PipeBody):
                             if isinstance(function_call.body, PipedCall) or isinstance(
                                 function_call.body, FuncCall
                             ):
-                                agg_str += f"{ast_to_sql(function_call.body, roots, symbol_table, verbose)} as {name},"
+                                agg_str += f"{ast_to_sql(function_call.body, roots, symbol_table, replace_tables, verbose)} as {name},"
                             else:
                                 agg_str += f"{function_call} as {name},"
 
                         elif isinstance(function_call, PipedCall):
-                            agg_str += f"{ast_to_sql(function_call, roots, symbol_table, verbose)} as {name},"
+                            agg_str += f"{ast_to_sql(function_call, roots, symbol_table, replace_tables, verbose)} as {name},"
                         else:
                             raise PRQLException(
                                 f"Unknown type for aggregate body {type(line)}, {str(line)}"
@@ -958,7 +1083,7 @@ def ast_to_sql(
                         piped = line
                         # piped.func_body.parm1 = replace_variables(str(piped.parm1), symbol_table)
                         # piped.func_body.func_args = replace_variables(str(piped.func_args), symbol_table)
-                        agg_str += f"{ast_to_sql(piped.func_body, roots, symbol_table, verbose)} as {piped.parm1}_{piped.func_body.name},"
+                        agg_str += f"{ast_to_sql(piped.func_body, roots, symbol_table, replace_tables, verbose)} as {piped.parm1}_{piped.func_body.name},"
                     elif isinstance(line, str):
                         agg_str += f"{line},"
                     elif isinstance(line, PipeBody):
@@ -967,6 +1092,7 @@ def ast_to_sql(
                                 line.body,
                                 roots,
                                 symbol_table=symbol_table,
+                                replace_tables=replace_tables,
                                 verbose=verbose,
                             )
                             + ","
@@ -977,7 +1103,7 @@ def ast_to_sql(
                         )
                 i += 1
 
-            agg_str = replace_tables(agg_str)
+            agg_str = replace_tables_func(agg_str)
             agg_str = agg_str.rstrip(",").lstrip(",")
         if take:
             limit_str = str(take)
@@ -988,9 +1114,13 @@ def ast_to_sql(
                     for function_call in filter.fields:
                         if function_call.val is not None:
                             filter_str += (
-                                replace_tables(
+                                replace_tables_func(
                                     ast_to_sql(
-                                        function_call.val, roots, symbol_table, verbose
+                                        function_call.val,
+                                        roots,
+                                        symbol_table,
+                                        replace_tables,
+                                        verbose,
                                     )
                                 )
                                 + " AND "
@@ -1000,11 +1130,11 @@ def ast_to_sql(
         if wheres_from_derives:
             for d in wheres_from_derives:
                 for line in d.fields:
-                    derives_str += f"{replace_tables(ast_to_sql(line.expression, roots, symbol_table, verbose))} as {line.name} ,"
+                    derives_str += f"{replace_tables_func(ast_to_sql(line.expression, roots, symbol_table, replace_tables, verbose))} as {line.name} ,"
             derives_str = "," + derives_str.rstrip(",")
 
         if sort:
-            order_by_str = f"ORDER BY {sort}"
+            order_by_str = f"ORDER BY {replace_tables_func(str(sort))}"
 
         if not select_str and not agg_str:
             select_str = "*"
@@ -1034,10 +1164,24 @@ def ast_to_sql(
                 order_by_str,
                 limit_str,
             )
-        sql = f"SELECT {select_str} {agg_str} {derives_str} FROM {from_str} {join_str} WHERE {filter_str} {group_by_str} {havings_str} {order_by_str} {limit_str}"
+        sql = f"SELECT {select_str} {agg_str} {derives_str} FROM {from_str} {join_str} WHERE {filter_str} {group_by_str} {havings_str} {order_by_str} {limit_str} {to_str}"
         if verbose:
             print("\t" + sql)
         return sql
+    elif isinstance(rule, BinaryExpression):
+        left = replace_variables(
+            ast_to_sql(rule.left, roots, symbol_table, replace_tables, verbose),
+            symbol_table,
+        )
+        if rule.right:
+            left += rule.operator + replace_variables(
+                ast_to_sql(rule.right, roots, symbol_table, replace_tables, verbose),
+                symbol_table,
+            )
+        if isinstance(rule, ParensExpression):
+            return "(" + left + ")"
+        else:
+            return left
     elif isinstance(rule, Expression):
         expr = rule
         msg = ""
@@ -1053,11 +1197,15 @@ def ast_to_sql(
                     if i + 1 < upper and isinstance(expr.statements[i + 1], Token):
                         fields.parm2 = expr.statements[i + 1]
                         i += 1
-                    msg += ast_to_sql(fields, roots, symbol_table, verbose)
+                    msg += ast_to_sql(
+                        fields, roots, symbol_table, replace_tables, verbose
+                    )
                 else:
-                    msg += ast_to_sql(fields, roots, symbol_table, verbose)
+                    msg += ast_to_sql(
+                        fields, roots, symbol_table, replace_tables, verbose
+                    )
             else:
-                msg += ast_to_sql(fields, roots, symbol_table, verbose)
+                msg += ast_to_sql(fields, roots, symbol_table, replace_tables, verbose)
             # else:
             #     raise PRQLException(f'Unknown type for expression {type(fields)}, {str(fields)}')
             i += 1
@@ -1072,7 +1220,7 @@ def ast_to_sql(
             pipe.func_body.parm2 = pipe.func_body.parm1
             pipe.func_body.parm1 = pipe.parm1
 
-        msg += ast_to_sql(pipe.func_body, roots, symbol_table, verbose)
+        msg += ast_to_sql(pipe.func_body, roots, symbol_table, replace_tables, verbose)
 
         return msg
     elif isinstance(rule, FuncCall):
@@ -1090,7 +1238,13 @@ def ast_to_sql(
                 if table.name == val:
                     return (
                         "("
-                        + ast_to_sql(table.value_body, roots, symbol_table, verbose)
+                        + ast_to_sql(
+                            table.value_body,
+                            roots,
+                            symbol_table,
+                            replace_tables,
+                            verbose,
+                        )
                         + ")"
                     )
         # TODO , should we use replace_variables here ?
@@ -1105,8 +1259,8 @@ def ast_to_sql(
     elif isinstance(rule, Name):
         return str(rule)
     elif isinstance(rule, FilterLine):
-        return ast_to_sql(rule.val, roots, symbol_table, verbose)
+        return ast_to_sql(rule.val, roots, symbol_table, replace_tables, verbose)
     elif isinstance(rule, DeriveBody):
-        return ast_to_sql(rule.val, roots, symbol_table, verbose)
+        return ast_to_sql(rule.val, roots, symbol_table, replace_tables, verbose)
     else:
         raise PRQLException(f"No sql for {type(rule)}")
